using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Data.Odbc;
using Microsoft.Win32;

namespace FenixOPCBarGraph
{
    internal partial class DataLoggingEditorForm : Form
    {
        public DataLoggingConfig loggingConfig = null;       
        
        [NonSerialized]
        private OdbcConnection conn = null;

        
        public DataLoggingEditorForm()
        {
            InitializeComponent();
        }

        private void DataLoggingEditorForm_Load(object sender, EventArgs e)
        {
            // Load previous configuration
            if ((loggingConfig != null) && (loggingConfig.ConnString != null))
            {
                OdbcConnectionStringBuilder csb = new OdbcConnectionStringBuilder(loggingConfig.ConnString);
                uxDSNComboBox.Items.Add(csb.Dsn);
                if (uxDSNComboBox.Items.Count > 0) uxDSNComboBox.SelectedIndex = 0;

                if (csb.ContainsKey("Uid")) uxUsernameTextBox.Text = csb["Uid"].ToString();
                if (csb.ContainsKey("Pwd")) uxPasswordTextBox.Text = csb["Pwd"].ToString();

                conn = new OdbcConnection(loggingConfig.ConnString);
                try
                {
                    conn.Open();

                    DataTable tables = conn.GetSchema(OdbcMetaDataCollectionNames.Tables);
                    uxTableComboBox.DataSource = tables;
                    uxTableComboBox.DisplayMember = "TABLE_NAME";

                    uxBarGraphIDTextBox.Text = loggingConfig.BarGraphID;
                    uxTableComboBox.Text = loggingConfig.Table;
                    uxBarGraphIdComboBox.Text = loggingConfig.BarGraphIDColumn;
                    uxBarIdComboBox.Text = loggingConfig.BarIDColumn;
                    uxBarValueComboBox.Text = loggingConfig.TagValueColumn;
                    uxTimestampComboBox.Text = loggingConfig.TimestampColumn;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Connection error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                uxBarGraphIDTextBox.Text = "FenixOPCBarGraph";
            }
        }
        
        private void uxOKButton_Click(object sender, EventArgs e)
        {
            if (uxCreateTableCheckBox.Checked)
            {
                try
                {
                    String tableName = "log" + DateTime.Now.ToString("ddMMyyyyHHmmss");
                    String command = "CREATE TABLE " + tableName + " (bargraphid VARCHAR(100), barid VARCHAR(100), barvalue NUMBER, valuetimestamp TIMESTAMP)";
                    OdbcCommand query = new OdbcCommand(command, conn);
                    query.ExecuteNonQuery();

                    loggingConfig = new DataLoggingConfig();

                    // connection settings
                    OdbcConnectionStringBuilder csb = new OdbcConnectionStringBuilder(conn.ConnectionString);
                    if (uxPasswordTextBox.Text != null) csb.Add("Pwd", uxPasswordTextBox.Text);
                    loggingConfig.ConnString = csb.ConnectionString;

                    loggingConfig.Table = tableName;
                    loggingConfig.BarGraphID = uxBarGraphIDTextBox.Text;
                    loggingConfig.BarGraphIDColumn = "bargraphid";
                    loggingConfig.BarIDColumn = "barid";
                    loggingConfig.TagValueColumn = "barvalue";
                    loggingConfig.TimestampColumn = "valuetimestamp";
                }
                catch(Exception ex)
                {
                    MessageBox.Show(ex.Message, "Database Error", MessageBoxButtons.OK, MessageBoxIcon.Error);;
                    return;
                }
            }
            else
            if ((conn != null) &&
                (uxBarGraphIdComboBox.Text != "") &&
                (uxBarIdComboBox.Text != "") &&
                (uxBarValueComboBox.Text != "") &&
                (uxTimestampComboBox.Text != ""))
            {
                loggingConfig = new DataLoggingConfig();

                // connection settings
                OdbcConnectionStringBuilder csb = new OdbcConnectionStringBuilder(conn.ConnectionString);
                if (uxPasswordTextBox.Text != null) csb.Add("Pwd", uxPasswordTextBox.Text);
                loggingConfig.ConnString = csb.ConnectionString;

                // table and column binding
                loggingConfig.Table = uxTableComboBox.Text;
                loggingConfig.BarGraphID = uxBarGraphIDTextBox.Text;
                loggingConfig.BarGraphIDColumn = uxBarGraphIdComboBox.Text;
                loggingConfig.BarIDColumn = uxBarIdComboBox.Text;
                loggingConfig.TagValueColumn = uxBarValueComboBox.Text;
                loggingConfig.TimestampColumn = uxTimestampComboBox.Text;
            }
            else
            {
                MessageBox.Show("Assign all columns!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            this.DialogResult = DialogResult.OK;
        }

        private void uxCancelButton_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
        }

        private void uxRefreshButton_Click(object sender, EventArgs e)
        {
            uxDSNComboBox.Items.Clear();
            uxTableComboBox.Text = "";
            uxBarGraphIdComboBox.Text = "";
            uxBarIdComboBox.Text = "";
            uxBarValueComboBox.Text = "";
            uxTimestampComboBox.Text = "";

            // System DSN List
            RegistryKey dsnList = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\ODBC\ODBC.INI\ODBC Data Sources");

            if (dsnList != null)
            {
                foreach (string dsn in dsnList.GetValueNames())
                {
                    uxDSNComboBox.Items.Add(dsn);
                }
            }
            
            // User DSN List
            dsnList = Registry.CurrentUser.OpenSubKey(@"SOFTWARE\ODBC\ODBC.INI\ODBC Data Sources");

            if (dsnList != null)
            {
                foreach (string dsn in dsnList.GetValueNames())
                {
                    uxDSNComboBox.Items.Add(dsn);
                }
            }
            if (uxDSNComboBox.Items.Count > 0) uxDSNComboBox.SelectedIndex = 0;
        }

        private void uxConnectButton_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;
            
            OdbcConnectionStringBuilder csb = new OdbcConnectionStringBuilder();
            
            csb.Dsn = uxDSNComboBox.Text;
            if (uxUsernameTextBox.Text != "") csb.Add("Uid", uxUsernameTextBox.Text);
            if (uxPasswordTextBox.Text != "") csb.Add("Pwd", uxPasswordTextBox.Text);

            conn = new OdbcConnection(csb.ConnectionString);

            try
            {
                conn.Open();

                DataTable tables = conn.GetSchema(OdbcMetaDataCollectionNames.Tables);
                uxTableComboBox.DataSource = tables;
                uxTableComboBox.DisplayMember = "TABLE_NAME";

                if (uxTableComboBox.Items.Count > 0) uxTableComboBox.SelectedIndex = 0;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Connection error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            this.Cursor = Cursors.Default;
        }

        private void uxTableComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Get column names from selected table and populate ComboBoxes
            String[] columnFilter = new String[] { null, null, uxTableComboBox.Text };
            DataTable columns = conn.GetSchema(OdbcMetaDataCollectionNames.Columns, columnFilter);
            
            uxBarGraphIdComboBox.DataSource = columns;
            uxBarGraphIdComboBox.BindingContext = new BindingContext();
            uxBarGraphIdComboBox.DisplayMember = "COLUMN_NAME";

            uxBarIdComboBox.DataSource = columns;
            uxBarIdComboBox.BindingContext = new BindingContext();
            uxBarIdComboBox.DisplayMember = "COLUMN_NAME";

            uxBarValueComboBox.DataSource = columns;
            uxBarValueComboBox.BindingContext = new BindingContext();
            uxBarValueComboBox.DisplayMember = "COLUMN_NAME";

            uxTimestampComboBox.DataSource = columns;
            uxTimestampComboBox.BindingContext = new BindingContext();
            uxTimestampComboBox.DisplayMember = "COLUMN_NAME";
        }

        private void DataLoggingEditorForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if ((conn != null) && (conn.State != ConnectionState.Closed)) conn.Close();
        }

        private void uxCreateTableCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (uxCreateTableCheckBox.Checked)
            {
                uxTableComboBox.Enabled = false;
                uxColumnGroupBox.Enabled = false;
            }
            else
            {
                uxTableComboBox.Enabled = true;
                uxColumnGroupBox.Enabled = true;
            }
        }
    }
}