using System;
using System.Collections.Generic;
using System.Text;

namespace FenixOPCBarGraph
{
    /// <summary>
    /// OPC Configuration parameters
    /// </summary>
    [Serializable]
    public class OPCTagConfig
    {
        private String host = null;
        private String group = null;
        private String tagName = null;

        #region Properties

        /// <summary>
        /// Computer name
        /// </summary>
        /// <remarks>
        /// Use the <b>Host</b> property to specify computer name for OPC Tag source. 
        /// </remarks>
        public String Host
        {
            get{return host;}
            set{host = value;}
        }

        /// <summary>
        /// OPC group on server
        /// </summary>
        /// <remarks>
        /// Use the <b>Group</b> property to specify OPC Group for OPC Tag source. 
        /// </remarks>
        public String Group
        {
            get { return group; }
            set { group = value; }
        }

        /// <summary>
        /// OPC tag name
        /// </summary>
        /// <remarks>
        /// Use the <b>TagName</b> property to specify OPC Tag source.
        /// </remarks>
        public String TagName
        {
            get { return tagName; }
            set { tagName = value; }
        }

        #endregion


        /// <summary>
        /// Short text for display in property editor
        /// </summary>
        /// <remarks>
        /// This method just overrides default functionality of <c>ToString()</c> method.
        /// <note>
        /// Be careful if overriding this method because this can have unwanted consequences for PropertyEditor functionality.
        /// </note>
        /// </remarks>
        /// <returns>"OPCConfig" string</returns>
        public override string ToString()
        {
            return "(OPCConfig)";
        }
    }
}
