using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace FenixOPCBarGraph
{
    internal partial class OPCTagEditorForm : Form
    {
        public OPCTagConfig tagConfig = null;
        private OPCTagConfig tmpConfig = new OPCTagConfig();

        
        public OPCTagEditorForm()
        {
            InitializeComponent();
        }

        private void OPCTagEditorForm_Load(object sender, EventArgs e)
        {
            refreshServers();

            // Get previous selected configuration
            if ((tagConfig != null) && (tagConfig.TagName != null))
            {
                tmpConfig = tagConfig;
                
                // Selected Host
                TreeNode serverNode = null;
                foreach (TreeNode tn in uxOPCServersTreeView.Nodes)
                {
                    if (tn.Text == tmpConfig.Host)
                    {
                        serverNode = tn;
                        break;
                    }
                }
                               
                // Selected Group
                getGroups("", serverNode);
                serverNode.Expand();

                // Selected Tag
                foreach (String tag in opcSystemsComponent.GetTagNames(tagConfig.Group, tagConfig.Host))
                {
                    uxOPCTagsListBox.Items.Add(tag);
                }
                uxOPCTagsListBox.SelectedItem = tagConfig.TagName;
            }
        }
        
        private void uxOK_Click(object sender, EventArgs e)
        {
            if ((tmpConfig != null) && (tmpConfig.TagName != "N/A"))
            {
                tagConfig = tmpConfig;
            }
            else tagConfig = new OPCTagConfig();

            DialogResult = DialogResult.OK;
        }

        private void uxCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
        }

        private void uxRefreshServerList_Click(object sender, EventArgs e)
        {
            refreshServers();
        }

        private void uxOPCServersTreeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            uxOPCTagsListBox.Items.Clear();
            uxOPCTagsListBox.Items.Add("N/A");

            // Get list of Groups
            if (e.Node.Level == 0)
            {
                tmpConfig.Host = e.Node.Text;
                tmpConfig.Group = "";

                e.Node.Nodes.Clear();
                getGroups("", e.Node);
                e.Node.Expand();
            }

            // Get selected Group
            if (e.Node.Level > 0)
            {
                tmpConfig.Group = e.Node.FullPath.Replace(tmpConfig.Host + ".", "");
            }

            // Get Tags
            foreach (String tag in opcSystemsComponent.GetTagNames(tmpConfig.Group, tmpConfig.Host))
            {
                uxOPCTagsListBox.Items.Add(tag);
            }
        }

        private void uxOPCTagsListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                tmpConfig.TagName = (String)uxOPCTagsListBox.Items[uxOPCTagsListBox.SelectedIndex];
            }
            catch (Exception err)
            {
                MessageBox.Show("Error: " + err.Message);
            }
        }

        private void refreshServers()
        {
            this.Cursor = Cursors.WaitCursor;

            uxOPCServersTreeView.Nodes.Clear();
            uxOPCTagsListBox.Items.Clear();
            uxOPCTagsListBox.Items.Add("N/A");

            // Get Hosts
            foreach (String host in opcSystemsComponent.GetNetworkNodes())
            {
                uxOPCServersTreeView.Nodes.Add(host);
            }

            this.Cursor = Cursors.Default;
        }

        private void getGroups(String parentGroup, TreeNode parentNode)
        {
            this.Cursor = Cursors.WaitCursor;

            foreach(String group in opcSystemsComponent.GetGroupNames(parentGroup, tmpConfig.Host))
            {
                TreeNode currentNode = parentNode.Nodes.Add(group);
                currentNode.SelectedImageIndex = currentNode.ImageIndex = 1;
                
                getGroups(group, currentNode);
            }

            this.Cursor = Cursors.Default;
        }
    }
}